@@ -25,6 +25,7 @@
 
 #include <KColorUtils>
 #include <KColorScheme>
+#include <KWindowSystem>
 
 #if OXYGEN_USE_KDE4
 #include <KGlobalSettings>
@@ -89,8 +90,6 @@ namespace Oxygen
         //____________________________________________________________________
     void Helper::invalidateCaches()
     {
-        _slabCache.clear();
-        _slabSunkenCache.clear();
         _decoColorCache.clear();
         _lightColorCache.clear();
         _darkColorCache.clear();
@@ -108,8 +107,6 @@ namespace Oxygen
     {
 
         // assign value
-        _slabCache.setMaxCacheSize( value );
-        _slabSunkenCache.setMaxCost( value );
         _backgroundCache.setMaxCost( value );
         _dotCache.setMaxCost( value );
 
@@ -186,15 +183,20 @@ namespace Oxygen
     {
 
         const quint64 key( colorKey(baseColor) );
-        QPixmap* pixmap( _dotCache.object( key ) );
+        QPixmap pixmap;
 
-        if( !pixmap )
+        if( QPixmap* cachedPixmap = _dotCache.object( key ) )
         {
-            pixmap = new QPixmap( highDpiPixmap( 4 ) );
-            pixmap->fill( Qt::transparent );
+
+            pixmap = *cachedPixmap;
+
+        } else {
+
+            pixmap = highDpiPixmap( 4 );
+            pixmap.fill( Qt::transparent );
             const qreal diameter( 1.8 );
 
-            QPainter painter( pixmap );
+            QPainter painter( &pixmap );
             painter.setRenderHint( QPainter::Antialiasing );
             painter.setPen( Qt::NoPen );
 
@@ -210,14 +212,14 @@ namespace Oxygen
             painter.end();
 
             // store in cache
-            _dotCache.insert( key, pixmap );
+            _dotCache.insert( key, new QPixmap( pixmap ) );
 
         }
 
         p->save();
         p->translate( point - QPoint( 1,1 ) );
         p->setRenderHint( QPainter::Antialiasing );
-        p->drawPixmap( QPoint( 0,0 ), *pixmap );
+        p->drawPixmap( QPoint( 0,0 ), pixmap );
         p->restore();
 
     }
@@ -264,149 +266,145 @@ namespace Oxygen
     }
 
     //____________________________________________________________________
-    const QColor& Helper::backgroundRadialColor( const QColor& color )
+    QColor Helper::backgroundRadialColor( const QColor& color )
     {
         const quint64 key( colorKey(color) );
-        QColor* out( _backgroundRadialColorCache.object( key ) );
-        if( !out )
-        {
-            if( lowThreshold( color ) ) out = new QColor( KColorScheme::shade( color, KColorScheme::LightShade, 0.0 ) );
-            else if( highThreshold( color ) ) out = new QColor( color );
-            else out = new QColor( KColorScheme::shade( color, KColorScheme::LightShade, _bgcontrast ) );
-            _backgroundRadialColorCache.insert( key, out );
-        }
+        if( QColor* cachedColor = _backgroundRadialColorCache.object( key ) )
+        { return *cachedColor; }
 
-        return *out;
+        QColor out;
+        if( lowThreshold( color ) ) out = KColorScheme::shade( color, KColorScheme::LightShade, 0.0 );
+        else if( highThreshold( color ) ) out = color;
+        else out = KColorScheme::shade( color, KColorScheme::LightShade, _bgcontrast );
+
+        _backgroundRadialColorCache.insert( key, new QColor( out ) );
+
+        return out;
     }
 
     //_________________________________________________________________________
-    const QColor& Helper::backgroundTopColor( const QColor& color )
+    QColor Helper::backgroundTopColor( const QColor& color )
     {
         const quint64 key( colorKey(color) );
-        QColor* out( _backgroundTopColorCache.object( key ) );
-        if( !out )
-        {
-            if( lowThreshold( color ) ) out = new QColor( KColorScheme::shade( color, KColorScheme::MidlightShade, 0.0 ) );
-            else {
-                const qreal my( KColorUtils::luma( KColorScheme::shade( color, KColorScheme::LightShade, 0.0 ) ) );
-                const qreal by( KColorUtils::luma( color ) );
-                out = new QColor( KColorUtils::shade( color, ( my - by ) * _bgcontrast ) );
-            }
+        if( QColor* cachedColor = _backgroundTopColorCache.object( key ) )
+        { return *cachedColor; }
 
-            _backgroundTopColorCache.insert( key, out );
+        QColor out;
+        if( lowThreshold( color ) ) out = KColorScheme::shade( color, KColorScheme::MidlightShade, 0.0 );
+        else {
+            const qreal my( KColorUtils::luma( KColorScheme::shade( color, KColorScheme::LightShade, 0.0 ) ) );
+            const qreal by( KColorUtils::luma( color ) );
+            out = KColorUtils::shade( color, ( my - by ) * _bgcontrast );
         }
 
-        return *out;
+        _backgroundTopColorCache.insert( key, new QColor( out ) );
 
+        return out;
     }
 
     //_________________________________________________________________________
-    const QColor& Helper::backgroundBottomColor( const QColor& color )
+    QColor Helper::backgroundBottomColor( const QColor& color )
     {
         const quint64 key( colorKey(color) );
-        QColor* out( _backgroundBottomColorCache.object( key ) );
-        if( !out )
-        {
-            const QColor midColor( KColorScheme::shade( color, KColorScheme::MidShade, 0.0 ) );
-            if( lowThreshold( color ) ) out = new QColor( midColor );
-            else {
+        if( QColor* cachedColor = _backgroundBottomColorCache.object( key ) )
+        { return *cachedColor; }
 
-                const qreal by( KColorUtils::luma( color ) );
-                const qreal my( KColorUtils::luma( midColor ) );
-                out = new QColor( KColorUtils::shade( color, ( my - by ) * _bgcontrast ) );
+        QColor out;
+        const QColor midColor( KColorScheme::shade( color, KColorScheme::MidShade, 0.0 ) );
+        if( lowThreshold( color ) ) out = midColor;
+        else {
 
-            }
+            const qreal by( KColorUtils::luma( color ) );
+            const qreal my( KColorUtils::luma( midColor ) );
+            out = KColorUtils::shade( color, ( my - by ) * _bgcontrast );
 
-            _backgroundBottomColorCache.insert( key, out );
         }
 
-        return *out;
+        _backgroundBottomColorCache.insert( key, new QColor( out ) );
+
+        return out;
 
     }
 
     //____________________________________________________________________
-    const QColor& Helper::calcLightColor( const QColor& color )
+    QColor Helper::calcLightColor( const QColor& color )
     {
         const quint64 key( colorKey(color) );
-        QColor* out( _lightColorCache.object( key ) );
-        if( !out )
-        {
-            out = new QColor( highThreshold( color ) ? color: KColorScheme::shade( color, KColorScheme::LightShade, _contrast ) );
-            _lightColorCache.insert( key, out );
-        }
+        if( QColor* cachedColor = _lightColorCache.object( key ) )
+        { return *cachedColor; }
 
-        return *out;
+        QColor out = highThreshold( color ) ? color: KColorScheme::shade( color, KColorScheme::LightShade, _contrast );
+        _lightColorCache.insert( key, new QColor( out ) );
+
+        return out;
 
     }
 
     //____________________________________________________________________
-    const QColor& Helper::calcDarkColor( const QColor& color )
+    QColor Helper::calcDarkColor( const QColor& color )
     {
         const quint64 key( colorKey(color) );
-        QColor* out( _darkColorCache.object( key ) );
-        if( !out )
-        {
-            out = new QColor( ( lowThreshold( color ) ) ?
-                KColorUtils::mix( calcLightColor( color ), color, 0.3 + 0.7 * _contrast ):
-                KColorScheme::shade( color, KColorScheme::MidShade, _contrast ) );
-            _darkColorCache.insert( key, out );
-        }
+        if( QColor* cachedColor = _darkColorCache.object( key ) )
+        { return *cachedColor; }
 
-        return *out;
+        QColor out = ( lowThreshold( color ) ) ?
+            KColorUtils::mix( calcLightColor( color ), color, 0.3 + 0.7 * _contrast ):
+            KColorScheme::shade( color, KColorScheme::MidShade, _contrast );
+        _darkColorCache.insert( key, new QColor( out ) );
+
+        return out;
     }
 
     //____________________________________________________________________
-    const QColor& Helper::calcShadowColor( const QColor& color )
+    QColor Helper::calcShadowColor( const QColor& color )
     {
 
         const quint64 key( colorKey(color) );
-        QColor* out( _shadowColorCache.object( key ) );
-        if( !out )
-        {
-            out = new QColor( ( lowThreshold( color ) ) ?
+        if( QColor* cachedColor = _shadowColorCache.object( key ) )
+        { return *cachedColor; }
+
+        QColor out = ( lowThreshold( color ) ) ?
             KColorUtils::mix( Qt::black, color, color.alphaF() ) :
             KColorScheme::shade(
-                KColorUtils::mix( Qt::black, color, color.alphaF() ),
-                KColorScheme::ShadowShade,
-                _contrast ) );
+            KColorUtils::mix( Qt::black, color, color.alphaF() ),
+            KColorScheme::ShadowShade,
+            _contrast );
 
-            // make sure shadow color has the same alpha channel as the input
-            out->setAlpha( color.alpha() );
+        // make sure shadow color has the same alpha channel as the input
+        out.setAlpha( color.alpha() );
 
-            // insert in cache
-            _shadowColorCache.insert( key, out );
-        }
+        // insert in cache
+        _shadowColorCache.insert( key, new QColor( out ) );
 
-        return *out;
+        return out;
 
     }
 
     //____________________________________________________________________
-    const QColor& Helper::backgroundColor( const QColor& color, qreal ratio )
+    QColor Helper::backgroundColor( const QColor& color, qreal ratio )
     {
 
         const quint64 key( ( colorKey(color) << 32 ) | int( ratio*512 ) );
-        QColor *out( _backgroundColorCache.object( key ) );
-        if( !out )
+        if( QColor* cachedColor = _backgroundColorCache.object( key ) )
+        { return *cachedColor; }
+
+        QColor out;
+        if( ratio < 0.5 )
         {
-            if( ratio < 0.5 )
-            {
 
-                const qreal a( 2.0*ratio );
-                out = new QColor( KColorUtils::mix( backgroundTopColor( color ), color, a ) );
+            const qreal a( 2.0*ratio );
+            out = KColorUtils::mix( backgroundTopColor( color ), color, a );
 
-            } else {
+        } else {
 
-                const qreal a( 2.0*ratio-1 );
-                out = new QColor( KColorUtils::mix( color, backgroundBottomColor( color ), a ) );
-
-            }
-
-            _backgroundColorCache.insert( key, out );
+            const qreal a( 2.0*ratio-1 );
+            out = KColorUtils::mix( color, backgroundBottomColor( color ), a );
 
         }
 
-        return *out;
+        _backgroundColorCache.insert( key, new QColor( out ) );
+
+        return out;
 
     }
 
@@ -415,71 +413,67 @@ namespace Oxygen
     QPixmap Helper::verticalGradient( const QColor& color, int height, int offset )
     {
         const quint64 key( ( colorKey(color) << 32 ) | height | 0x8000 );
-        QPixmap* pixmap( _backgroundCache.object( key ) );
 
-        if ( !pixmap )
-        {
-            pixmap = new QPixmap( 1, height );
-            pixmap->fill( Qt::transparent );
+        if( QPixmap* cachedPixmap = _backgroundCache.object( key ) )
+        { return *cachedPixmap; }
 
-            QLinearGradient gradient( 0, offset, 0, height );
-            gradient.setColorAt( 0.0, backgroundTopColor( color ) );
-            gradient.setColorAt( 0.5, color );
-            gradient.setColorAt( 1.0, backgroundBottomColor( color ) );
+        QPixmap pixmap( 1, height );
+        pixmap.fill( Qt::transparent );
 
-            QPainter painter( pixmap );
-            painter.fillRect( pixmap->rect(), gradient );
+        QLinearGradient gradient( 0, offset, 0, height );
+        gradient.setColorAt( 0.0, backgroundTopColor( color ) );
+        gradient.setColorAt( 0.5, color );
+        gradient.setColorAt( 1.0, backgroundBottomColor( color ) );
 
-            painter.end();
+        QPainter painter( &pixmap );
+        painter.fillRect( pixmap.rect(), gradient );
 
-            _backgroundCache.insert( key, pixmap );
-        }
+        painter.end();
 
-        return *pixmap;
+        _backgroundCache.insert( key, new QPixmap( pixmap ) );
+        return pixmap;
     }
 
     //____________________________________________________________________
     QPixmap Helper::radialGradient( const QColor& color, int width, int height )
     {
         const quint64 key( ( colorKey(color) << 32 ) | width | 0xb000 );
-        QPixmap* pixmap( _backgroundCache.object( key ) );
 
-        if ( !pixmap )
-        {
-            pixmap = new QPixmap( width, height );
-            pixmap->fill( Qt::transparent );
+        if( QPixmap* cachedPixmap = _backgroundCache.object( key ) )
+        { return *cachedPixmap; }
 
-            QRadialGradient gradient( 64, height-64, 64 );
-            QColor radialColor = backgroundRadialColor( color );
-            radialColor.setAlpha( 255 ); gradient.setColorAt( 0, radialColor );
-            radialColor.setAlpha( 101 ); gradient.setColorAt( 0.5, radialColor );
-            radialColor.setAlpha( 37 );  gradient.setColorAt( 0.75, radialColor );
-            radialColor.setAlpha( 0 );   gradient.setColorAt( 1, radialColor );
+        QPixmap pixmap( width, height );
+        pixmap.fill( Qt::transparent );
 
-            QPainter painter( pixmap );
-            painter.setWindow( 0, 0, 128, height );
-            painter.fillRect( QRect( 0,0,128,height ), gradient );
+        QRadialGradient gradient( 64, height-64, 64 );
+        QColor radialColor = backgroundRadialColor( color );
+        radialColor.setAlpha( 255 ); gradient.setColorAt( 0, radialColor );
+        radialColor.setAlpha( 101 ); gradient.setColorAt( 0.5, radialColor );
+        radialColor.setAlpha( 37 );  gradient.setColorAt( 0.75, radialColor );
+        radialColor.setAlpha( 0 );   gradient.setColorAt( 1, radialColor );
 
-            painter.end();
+        QPainter painter( &pixmap );
+        painter.setWindow( 0, 0, 128, height );
+        painter.fillRect( QRect( 0,0,128,height ), gradient );
 
-            _backgroundCache.insert( key, pixmap );
-        }
+        painter.end();
 
-        return *pixmap;
+        _backgroundCache.insert( key, new QPixmap( pixmap ) );
+
+        return pixmap;
     }
 
     //____________________________________________________________________________________
-    const QColor& Helper::decoColor( const QColor& background, const QColor& color )
+    QColor Helper::decoColor( const QColor& background, const QColor& color )
     {
         const quint64 key( ( colorKey(background) << 32 ) | colorKey(color) );
-        QColor* out( _decoColorCache.object( key ) );
-        if( !out )
-        {
-            out = new QColor( KColorUtils::mix( background, color, 0.8*( 1.0 + _contrast ) ) );
-            _decoColorCache.insert( key, out );
-        }
+        if( QColor* cachedColor = _decoColorCache.object( key ) )
+        { return *cachedColor; }
 
-        return *out;
+        QColor out = KColorUtils::mix( background, color, 0.8*( 1.0 + _contrast ) );
+        _decoColorCache.insert( key, new QColor( out ) );
+
+        return out;
     }
 
     //_______________________________________________________________________
@@ -760,163 +754,6 @@ namespace Oxygen
         p->restore();
     }
 
-    //________________________________________________________________________________________________________
-    TileSet *Helper::slab( const QColor& color, const QColor& glow, qreal shade, int size )
-    {
-        Oxygen::Cache<TileSet>::Value* cache( _slabCache.get( color ) );
-
-        const quint64 key( ( colorKey(glow) << 32 ) | ( quint64( 256.0 * shade ) << 24 ) | size );
-        TileSet *tileSet = cache->object( key );
-
-        if ( !tileSet )
-        {
-            QPixmap pixmap( highDpiPixmap( size*2 ) );
-            pixmap.fill( Qt::transparent );
-
-            QPainter painter( &pixmap );
-            painter.setRenderHints( QPainter::Antialiasing );
-            painter.setPen( Qt::NoPen );
-
-            const int fixedSize( 14*devicePixelRatio( pixmap ) );
-            painter.setWindow( 0, 0, fixedSize, fixedSize );
-
-            // draw all components
-            if( color.isValid() ) drawShadow( painter, calcShadowColor( color ), 14 );
-            if( glow.isValid() ) drawOuterGlow( painter, glow, 14 );
-            if( color.isValid() ) drawSlab( painter, color, shade );
-
-            painter.end();
-
-            tileSet = new TileSet( pixmap,
-                size, size,
-                size, size,
-                size-1, size,
-                2, 1 );
-
-            cache->insert( key, tileSet );
-
-        }
-        return tileSet;
-    }
-
-    //________________________________________________________________________________________________________
-    TileSet *Helper::slabSunken( const QColor& color, int size )
-    {
-        const quint64 key( colorKey(color) << 32 | size );
-        TileSet *tileSet = _slabSunkenCache.object( key );
-
-        if ( !tileSet )
-        {
-            QPixmap pixmap( highDpiPixmap( size*2 ) );
-            pixmap.fill( Qt::transparent );
-
-            QPainter painter( &pixmap );
-            painter.setRenderHints( QPainter::Antialiasing );
-            painter.setPen( Qt::NoPen );
-
-            const int fixedSize( 14*devicePixelRatio( pixmap ) );
-            painter.setWindow( 0, 0, fixedSize, fixedSize );
-
-            // shadow
-            painter.setCompositionMode( QPainter::CompositionMode_SourceOver );
-            drawInverseShadow( painter, calcShadowColor( color ), 3, 8, 0.0 );
-
-            // contrast pixel
-            {
-                QColor light( calcLightColor( color ) );
-                QLinearGradient blend( 0, 2, 0, 16 );
-                blend.setColorAt( 0.5, Qt::transparent );
-                blend.setColorAt( 1.0, light );
-
-                painter.setBrush( Qt::NoBrush );
-                painter.setPen( QPen( blend, 1 ) );
-                painter.drawRoundedRect( QRectF( 2.5, 2.5, 9, 9 ), 4.0, 4.0 );
-                painter.setPen( Qt::NoPen );
-            }
-
-
-            painter.end();
-
-            tileSet = new TileSet( pixmap, size, size, size, size, size-1, size, 2, 1 );
-
-            _slabSunkenCache.insert( key, tileSet );
-
-        }
-
-        return tileSet;
-
-    }
-
-    //________________________________________________________________________________________________________
-    void Helper::fillSlab( QPainter& painter, const QRect& rect, int size ) const
-    {
-        const qreal s( qreal( size ) * ( 3.6 + ( 0.5 * _slabThickness ) ) / 7.0 );
-        const QRectF r( QRectF( rect ).adjusted( s, s, -s, -s ) );
-        if( !r.isValid() ) return;
-
-        painter.drawRoundedRect( r, s/2, s/2 );
-    }
-
-    //________________________________________________________________________________________________________
-    void Helper::fillButtonSlab( QPainter& painter, const QRect& r, const QColor& color, bool sunken )
-    {
-
-        painter.save();
-        painter.setRenderHint( QPainter::Antialiasing );
-        painter.setPen( Qt::NoPen );
-
-        if( sunken && calcShadowColor( color ).value() > color.value() )
-        {
-
-            QLinearGradient innerGradient( 0, r.top(), 0, r.bottom() + r.height() );
-            innerGradient.setColorAt( 0.0, color );
-            innerGradient.setColorAt( 1.0, calcLightColor( color ) );
-            painter.setBrush( innerGradient );
-
-        } else if( sunken ) {
-
-
-            QLinearGradient innerGradient( 0, r.top() - r.height(), 0, r.bottom() );
-            innerGradient.setColorAt( 0.0, calcLightColor( color ) );
-            innerGradient.setColorAt( 1.0, color );
-            painter.setBrush( innerGradient );
-
-        } else {
-
-            QLinearGradient innerGradient( 0, r.top()-0.2*r.height(), 0, r.bottom()+ 0.4*r.height() );
-            innerGradient.setColorAt( 0.0, calcLightColor( color ) );
-            innerGradient.setColorAt( 0.6, color );
-            painter.setBrush( innerGradient );
-
-        }
-
-        fillSlab( painter, r );
-        painter.restore();
-
-    }
-
-    //________________________________________________________________________________________________________
-    void Helper::drawInverseShadow(
-        QPainter& painter, const QColor& color,
-        int pad, int size, qreal fuzz ) const
-    {
-
-        const qreal m( qreal( size )*0.5 );
-        const qreal offset( 0.8 );
-        const qreal k0( ( m-2 ) / qreal( m+2.0 ) );
-        QRadialGradient shadowGradient( pad+m, pad+m+offset, m+2 );
-        for ( int i = 0; i < 8; i++ )
-        {
-            // sinusoidal gradient
-            const qreal k1( ( qreal( 8 - i ) + k0 * qreal( i ) ) * 0.125 );
-            const qreal a( ( cos( 3.14159 * i * 0.125 ) + 1.0 ) * 0.25 );
-            shadowGradient.setColorAt( k1, alphaColor( color, a * _shadowGain ) );
-        }
-        shadowGradient.setColorAt( k0, alphaColor( color, 0.0 ) );
-        painter.setBrush( shadowGradient );
-        painter.drawEllipse( QRectF( pad-fuzz, pad-fuzz, size+fuzz*2.0, size+fuzz*2.0 ) );
-    }
-
     //____________________________________________________________________
     const QWidget* Helper::checkAutoFillBackground( const QWidget* w ) const
     {
@@ -987,7 +824,7 @@ namespace Oxygen
     {
         #if OXYGEN_HAVE_X11
         #if QT_VERSION >= 0x050000
-        static const bool s_isX11 = QX11Info::isPlatformX11();
+        static const bool s_isX11 = KWindowSystem::isPlatformX11();
         return s_isX11;
         #else
         return true;
@@ -1001,7 +838,7 @@ namespace Oxygen
     bool Helper::isWayland( void )
     {
         #if QT_VERSION >= 0x050000
-        static const bool s_isWayland = qApp->platformName().startsWith(QLatin1String("wayland"));
+        static const bool s_isWayland = KWindowSystem::isPlatformWayland();
         return s_isWayland;
         #else
         return false;
@@ -1038,53 +875,6 @@ namespace Oxygen
 
     #endif
 
-    //______________________________________________________________________________________
-    void Helper::drawSlab( QPainter& painter, const QColor& color, qreal shade )
-    {
-
-        const QColor light( KColorUtils::shade( calcLightColor( color ), shade ) );
-        const QColor base( alphaColor( light, 0.85 ) );
-        const QColor dark( KColorUtils::shade( calcDarkColor( color ), shade ) );
-
-        // bevel, part 1
-        painter.save();
-        const qreal y( KColorUtils::luma( base ) );
-        const qreal yl( KColorUtils::luma( light ) );
-        const qreal yd( KColorUtils::luma( dark ) );
-        QLinearGradient bevelGradient1( 0, 7, 0, 11 );
-        bevelGradient1.setColorAt( 0.0, light );
-        if ( y < yl && y > yd )
-        {
-            // no middle when color is very light/dark
-            bevelGradient1.setColorAt( 0.5, base );
-        }
-
-        bevelGradient1.setColorAt( 0.9, base );
-        painter.setBrush( bevelGradient1 );
-        painter.drawRoundedRect( QRectF( 3.0,3.0,8.0,8.0 ), 3.5, 3.5 );
-
-        // bevel, part 2
-        if ( _slabThickness > 0.0 )
-        {
-
-            QLinearGradient bevelGradient2( 0, 6, 0, 19 );
-            bevelGradient2.setColorAt( 0.0, light );
-            bevelGradient2.setColorAt( 0.9, base );
-            painter.setBrush( bevelGradient2 );
-            painter.drawEllipse( QRectF( 3.6,3.6,6.8,6.8 ) );
-
-        }
-
-        // inside mask
-        painter.setCompositionMode( QPainter::CompositionMode_DestinationOut );
-        painter.setBrush( Qt::black );
-
-        const qreal ic( 3.6 + 0.5*_slabThickness );
-        const qreal is( 14.0 - 2.0*ic );
-        painter.drawEllipse( QRectF( ic, ic, is, is ) );
-        painter.restore();
-    }
-
     //___________________________________________________________________________________________
     void Helper::drawShadow( QPainter& painter, const QColor& color, int size )
     {
